#include <ESPMegaDisplay.hpp>

bool ESPMegaDisplay::recieveSerialCommand(bool process){
    bool dataRecieved = false;
    // Read the serial buffer if available
    while(displayAdapter->available()) {
        rx_buffer[rx_buffer_index] = displayAdapter->read();
        rx_buffer_index++;
        // Check for overflow
        if(rx_buffer_index>=256){
            rx_buffer_index = 0;
        }
        if(process) this-> processSerialCommand();
        dataRecieved = true;
    }
    return dataRecieved;
}

bool ESPMegaDisplay::recieveSerialCommand(){
    return recieveSerialCommand(true);
}

void ESPMegaDisplay::processSerialCommand(){
    // Check if the rx buffer ended with stop bytes (0xFF 0xFF 0xFF)
    if(!payloadIsValid()) return;
    // The rx buffer ended with stop bytes
    // Check if the rx buffer is a push payload
    // The payload type is the first byte of the payload
    // 0x00 is invalid instruction
    // 0x01 is success
    // 0x65 is a touch event
    // 0x66 is a page report event
    if(rx_buffer[0]==0x65){
        processTouchPayload();
    }
    else if(rx_buffer[0]==0x66){
        processPageReportPayload();
    }
}

void ESPMegaDisplay::processTouchPayload() {
    if (rx_buffer_index != 4) return;
    // The second byte of the payload is the page number
    uint8_t page = rx_buffer[1];
    // The third byte of the payload is the component id
    uint8_t component = rx_buffer[2];
    // The fourth byte of the payload is the event
    uint8_t event = rx_buffer[3];
    // 0x01 is press, 0x00 is release
}

void ESPMegaDisplay::processPageReportPayload() {
    // The second byte of the payload is the page number
    this->currentPage = rx_buffer[1];
    if(pageChangeCallback!=NULL){
        pageChangeCallback(this->currentPage);
    }
    rx_buffer_index = 0;
}

void ESPMegaDisplay::sendStopBytes() {
    displayAdapter->write(0xFF);
    displayAdapter->write(0xFF);
    displayAdapter->write(0xFF);
}

void ESPMegaDisplay::sendCommand(char* command) {
    displayAdapter->print(command);
    sendStopBytes();
}

void ESPMegaDisplay::refreshPage() {
    if(this->currentPage==0){
        refreshInputPage();
    }
    else if(this->currentPage==1){
        refreshOutputPage();
    }
}

void ESPMegaDisplay::jumpToPage(int page) {
    this->displayAdapter->print("page ");
    this->displayAdapter->print(page);
    sendStopBytes();
}

void ESPMegaDisplay::setNumber(char* component, int value) {
    this->displayAdapter->print(component);
    this->displayAdapter->print("=");
    this->displayAdapter->print(value);
    sendStopBytes();
}

void ESPMegaDisplay::setString(char* component, char* value) {
    this->displayAdapter->print(component);
    this->displayAdapter->print("=\"");
    this->displayAdapter->print(value);
    this->displayAdapter->print("\"");
    sendStopBytes();
}

int ESPMegaDisplay::getNumber(char* component) {
    uint32_t start = millis();
    // Send the get command
    this->displayAdapter->print("get ");
    this->displayAdapter->print(component);
    sendStopBytes();
    // Wait for the response
    if(!waitForValidPayload(FETCH_TIMEOUT)) return 0;
    // The rx buffer is valid
    
}

bool ESPMegaDisplay::waitForValidPayload(uint32_t timeout) {
    uint32_t start = millis();
    // If the payload is not valid, keep reading the serial buffer until timeout
    while(!this->payloadIsValid()){
        if(millis()-start>timeout){
            return false;
        }
        recieveSerialCommand(false);
    }
    return true;
}

bool ESPMegaDisplay::payloadIsValid() {
    // Check if the rx buffer ended with stop bytes (0xFF 0xFF 0xFF)
    if(rx_buffer_index<3) return false;
    if(rx_buffer[rx_buffer_index-1]!=0xFF) return false;
    if(rx_buffer[rx_buffer_index-2]!=0xFF) return false;
    if(rx_buffer[rx_buffer_index-3]!=0xFF) return false;
    return true;
}